import zipfile
import os
import shutil
import pathlib
import sys


import platform
import subprocess


sys_os = platform.system()
is_mac = sys_os == 'Darwin'

# args = sys.argv
# working_dir = args[1]
# C:\Users\USERNAME\ladybug_tools
working_dir = pathlib.Path(__file__).parent.parent.absolute()


yak_rhino6_win = "C:\Program Files\Rhino 6\System\yak.exe"
yak_rhino6_win2 = "C:\Program Files\Rhinoceros 6\System\yak.exe"
yak_rhino6_mac = "/Applications/Rhinoceros.app/Contents/Resources/bin/yak"
yak_rhino7_win = "C:\Program Files\Rhino 7\System\yak.exe"
yak_rhino7_win2 = "C:\Program Files\Rhinoceros 7\System\yak.exe"
yak_rhino7_mac = "/Applications/Rhino 7.app/Contents/Resources/bin/yak"

installOrUninstall = 'install'
if len(sys.argv) > 1:
    installOrUninstall =  sys.argv[1]
isInstall = installOrUninstall == 'install'
print(f'Is install option: {isInstall}')


def is_rhino6():
    found = os.path.exists(yak_rhino6_win) or os.path.exists(yak_rhino6_win2) or os.path.exists(yak_rhino6_mac)
    if found:
        print('Found Rhino 6')
    return found


def is_rhino7():
    found = os.path.exists(yak_rhino7_win) or os.path.exists(yak_rhino7_win2) or os.path.exists(yak_rhino7_mac)
    if found:
        print('Found Rhino 7')
    return found


def get_rhino_package_folder(version):
    if is_mac:
        user_dir = os.environ['HOME']
        # '/Users/mingbo/Library/Application Support/McNeel/Rhinoceros/packages/7.0'
        path = os.path.join(user_dir, f'Library/Application Support/McNeel/Rhinoceros/packages/{version}')
        return path
    else:
        # windows
        # C:\Users\mingo\AppData\Roaming\McNeel\Rhinoceros\packages\7.0
        # C:\Users\mingo\AppData\Roaming\McNeel\Rhinoceros\packages\6.0
        user_dir = os.environ['APPDATA']
        path = os.path.join(user_dir, f'McNeel\Rhinoceros\packages\{version}')
        return path


def remove_old_version():
    # copy all ghuser files to Rhino's Grasshopper folder
    # Mac
    if is_mac:
        user_dir = os.environ['HOME']
        # '/Users/mingbo/Library/Application Support/McNeel/Rhinoceros/6.0/Plug-ins/Grasshopper (b45a29b1-4343-4035-989e-044e8580d9cf)/UserObjects/'
        grasshopper_tar_dir = os.path.join(user_dir, 'Library/Application Support/McNeel/Rhinoceros/6.0/Plug-ins')
        grasshopper_name = [f for f in os.listdir(grasshopper_tar_dir) if f.startswith('Grasshopper')][0]
        grasshopper_tar_dir = os.path.join(grasshopper_tar_dir, grasshopper_name, 'UserObjects')
        grasshopper_gha_tar_dir = os.path.join(grasshopper_tar_dir, grasshopper_name, 'Libraries')
    else:
        # windows
        # C:\Users\mingo\AppData\Roaming\Grasshopper\UserObjects
        user_dir = os.environ['APPDATA']
        grasshopper_tar_dir = os.path.join(user_dir, 'Grasshopper', 'UserObjects')
        grasshopper_gha_tar_dir = os.path.join(user_dir, 'Grasshopper', 'Libraries')


    # removing all existing files from old version
    old_grasshopper_plugin_dir = os.path.join(grasshopper_gha_tar_dir, 'pollination')
    if os.path.exists(old_grasshopper_plugin_dir):
        print(f'Removing old version from: {old_grasshopper_plugin_dir}')
        shutil.rmtree(old_grasshopper_plugin_dir)

    # remove old ghlink file
    ghlink = "pollination.ghlink"
    old_ghlink = os.path.join(grasshopper_gha_tar_dir, ghlink)
    # C:\Users\mingo\AppData\Roaming\Grasshopper\Libraries\pollination.ghlink
    if os.path.exists(old_ghlink):
        print(f'Removing old grasshopper plugin from: {grasshopper_gha_tar_dir}')
        os.remove(old_ghlink)


def copy_to_package_folder(plugin_src_dir, plugin_name, version):
    # skip if it is installed and source folder has been removed
    if not os.path.exists(plugin_src_dir):
        print(f'Failed to find: {plugin_src_dir}')
        return

    package_folders = []
    if is_rhino6():
        package_folder = get_rhino_package_folder(6.0)
        package_folders.append(package_folder)
        print(f'Installing to Rhino 6: {package_folder}')
    
    if is_rhino7():
        package_folder = get_rhino_package_folder(7.0)
        package_folders.append(package_folder)
        print(f'Installing to Rhino 7: {package_folder}')
    
    if len(package_folders) == 0:
        print('No rhino found')
        return
    
    for path in package_folders:
        if not os.path.exists(path):
            os.makedirs(path)
        # C:\Users\mingo\AppData\Roaming\McNeel\Rhinoceros\packages\7.0\PollinationGH
        plugin_dir = os.path.join(path, plugin_name)
        # remove existing first
        if os.path.exists(plugin_dir):
            shutil.rmtree(plugin_dir)
        
        plugin_dir_version = os.path.join(plugin_dir, version)
        shutil.copytree(plugin_src_dir, plugin_dir_version)
        print(f'Installed to: {plugin_dir_version}')
        create_manifest(plugin_dir, version)

    # remove plugins from installer folder
    shutil.rmtree(plugin_src_dir)
    # create an empty folder with a text file that points to Rhino's package path
    # create a different folder "plugins" instead of "plugin"
    new_plugin_src_dir = plugin_src_dir + "s" 
    os.makedirs(new_plugin_src_dir)
    create_link_text(new_plugin_src_dir, plugin_dir)

       

def create_link_text(path, package_path):
    # create a manifest.txt
    manifest = "pollination.txt"
    manifest = os.path.join(path, manifest)
    f = open(manifest, "w")
    f.write(package_path)
    f.close()

def create_manifest(path, version):
    # create a manifest.txt
    manifest = "manifest.txt"
    manifest = os.path.join(path, manifest)
    f = open(manifest, "w")
    f.write(version)
    f.close()


def remove_from_package_folder(plugin_name):
    package_folders = []
    if is_rhino6():
        package_folder = get_rhino_package_folder(6.0)
        package_folders.append(package_folder)
    
    if is_rhino7():
        package_folder = get_rhino_package_folder(7.0)
        package_folders.append(package_folder)
    
    for path in package_folders:
        if not os.path.exists(path):
            continue

        plugin_dir = os.path.join(path, plugin_name)
        # remove existing first
        if os.path.exists(plugin_dir):
            shutil.rmtree(plugin_dir)


if isInstall:
    remove_old_version()
    # plugin inclues all plugins: RH, GH, import.rhp, export.rhp etc
    # C:\Users\mingo\pollination\plugin to C:\Users\mingo\AppData\Roaming\McNeel\Rhinoceros\packages\7.0\Pollination
    plugin_dir = os.path.join(working_dir, 'plugin')
    copy_to_package_folder(plugin_dir, 'Pollination', '1.0.0')
else:
    remove_from_package_folder('Pollination')

